/**
 * 
 *  Copyright (C) 2007 Enterprise Distributed Technologies Ltd
 *
 *  www.enterprisedt.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Bug fixes, suggestions and comments should be should posted on 
 *  http://www.enterprisedt.com/forums/index.php
 *
 *  Change Log:
 *
 *    $Log: FileTransferOutputStream.java,v $
 *    Revision 1.2  2012-01-19 11:13:04  bruceb
 *    add getBytesTransferred
 *
 *    Revision 1.1  2007-12-18 07:52:06  bruceb
 *    2.0 changes
 *
 *
 */
package com.enterprisedt.net.ftp;

import java.io.OutputStream;

/**
 *  Super class of all output streams supported
 *
 *  @author      Bruce Blackshaw
 *  @version     $Revision: 1.2 $
 */
abstract public class FileTransferOutputStream extends OutputStream {

    /** 
     * Name of remote file being transferred
     */
    protected String remoteFile;
    
    /**
     * Number of bytes transferred
     */
    protected long size = 0;
    
    /**
     * Has the stream been closed?
     */
    protected boolean closed = false;
    
    /**
     * Get the name of the remote file (including one that may
     * have been generated by the server).
     * 
     * @return remote filename
     */
    public String getRemoteFile() {
        return remoteFile;
    }
    
    /**
     * Get the number of bytes transferred
     * 
     * @return long
     */
    public long getBytesTransferred() {
        return size;
    }
}
